@@ -82,7 +82,7 @@ class Face3D(Base2DIn3D):
                  '_is_convex', '_is_self_intersecting')
 
     def __init__(self, boundary, plane=None, holes=None, enforce_right_hand=True):
-        """Initilize Face3D."""
+        """Initialize Face3D."""
         # process the boundary and plane inputs
         self._boundary = self._check_vertices_input(boundary)
         if plane is not None:
@@ -232,7 +232,7 @@ class Face3D(Base2DIn3D):
 
         Args:
             side_count: An integer for the number of sides on the regular
-                polgygon. This number must be greater than 2.
+                polygon. This number must be greater than 2.
             radius: A number indicating the distance from the polygon's center
                 where the vertices of the polygon will lie.
                 The default is set to 1.
@@ -313,14 +313,14 @@ class Face3D(Base2DIn3D):
 
     @property
     def triangulated_mesh2d(self):
-        """A triagulated Mesh2D in the 2D space of the face's plane."""
+        """A triangulated Mesh2D in the 2D space of the face's plane."""
         if self._mesh2d is None:
             self._mesh2d = Mesh2D.from_polygon_triangulated(self.polygon2d)
         return self._mesh2d
 
     @property
     def triangulated_mesh3d(self):
-        """A triagulated Mesh3D of this face."""
+        """A triangulated Mesh3D of this face."""
         if self._mesh3d is None:
             _vert3d = tuple(self._plane.xy_to_xyz(_v) for _v in
                             self.triangulated_mesh2d.vertices)
@@ -539,7 +539,7 @@ class Face3D(Base2DIn3D):
     def is_geometrically_equivalent(self, face, tolerance):
         """Check whether a given face is geometrically equivalent to this Face.
 
-        Geometrical equivalence is definied as being coplananar with this face,
+        Geometrical equivalence is defined as being coplanar with this face,
         having the same number of vertices, and having each vertex map-able between
         the faces. Clockwise relationships do not have to match nor does the normal
         direction of the face.  However, all other properties must be matching to
@@ -589,7 +589,7 @@ class Face3D(Base2DIn3D):
     def is_centered_adjacent(self, face, tolerance):
         """Check whether a given face is centered adjacent with this Face.
 
-        Centered adjacency is definied as sharing the same center point as this face,
+        Centered adjacency is defined as sharing the same center point as this face,
         sharing the same area, and being next to one another to within the tolerance.
 
         This is useful for identifying matching surfaces when you want to quickly
@@ -645,24 +645,13 @@ class Face3D(Base2DIn3D):
             return False
 
         # if it is, convert sub-face to a polygon in this face's plane
-        verts2d = tuple(self.plane.xyz_to_xy(_v) for _v in face.vertices)
-        sub_poly = Polygon2D(verts2d)
-
-        if not self.has_holes:
-            return self.polygon2d.is_polygon_inside(sub_poly)
-        else:
-            if not self.boundary_polygon2d.is_polygon_inside(sub_poly):
-                return False
-            for hole_poly in self.hole_polygon2d:
-                if not hole_poly.is_polygon_outside(sub_poly):
-                    return False
-            return True
+        return self._is_sub_face(face)
 
     def is_point_on_face(self, point, tolerance):
         """Check whether a given point is on this face.
 
         This includes both a check to be sure that the point is in the plane of this
-        face and a chek to ensure that point lies in the boundary of the face.
+        face and a check to ensure that point lies in the boundary of the face.
 
         Args:
             face: Another face for which sub-face equivalency will be tested.
@@ -685,7 +674,7 @@ class Face3D(Base2DIn3D):
         it is assumed that there is likely a check for planarity before the face
         is created (ie. in CAD software where the face likely originates from).
         This method is intended for quality control checking when the origin of
-        face geometry is unkown or is known to come from a place where no
+        face geometry is unknown or is known to come from a place where no
         planarity check was performed.
 
         Args:
@@ -801,7 +790,7 @@ class Face3D(Base2DIn3D):
         """
         _verts = self._reflect(self.vertices, normal, origin)
         _new_face = self._face_transform_reflect(
-                _verts, self.plane.reflect(normal, origin))
+            _verts, self.plane.reflect(normal, origin))
         if self._holes is not None:
             _new_face._boundary = self._reflect(self._boundary, normal, origin)
             _new_face._holes = tuple(self._reflect(hole, normal, origin)
@@ -890,8 +879,8 @@ class Face3D(Base2DIn3D):
             return _plane_int
         return None
 
-    def get_mesh_grid(self, x_dim, y_dim=None, offset=None, flip=False,
-                      generate_centroids=True):
+    def mesh_grid(self, x_dim, y_dim=None, offset=None, flip=False,
+                  generate_centroids=True):
         """Get a gridded Mesh3D over this face.
 
         This method generates a mesh grid over the domain of the face
@@ -957,8 +946,7 @@ class Face3D(Base2DIn3D):
 
         return grid_mesh3d
 
-    def countour_by_number(self, contour_count, direction_vector=Vector2D(0, 1),
-                           flip_side=False, tolerance=0):
+    def countour_by_number(self, contour_count, direction_vector, flip_side, tolerance):
         """Generate a list of LineSegment3D objects contouring the face.
 
         Args:
@@ -968,13 +956,14 @@ class Face3D(Base2DIn3D):
                 are generated. This 2D vector will be interpreted into a 3D vector
                 within the plane of this Face. (0, 1) will usually generate
                 horizontal contours in 3D space, (1, 0) will generate vertical
-                contours, and (1, 1) will generate diagonal contours. Default: (0, 1).
+                contours, and (1, 1) will generate diagonal contours. Recommended
+                value is Vactor2D(0, 1).
             flip_side: Boolean to note whether the side the contours start from
-                should be flipped. Default is False to have contours on top or right.
+                should be flipped. Recommended value is False to have contours
+                on top or right.
                 Setting to True will start contours on the bottom or left.
             tolerance: An optional value to remove any contours with a length less
-                than the tolerance. Default is 0, which will include all contours
-                no matter how small.
+                than the tolerance.
         """
         # interpret the 2D direction_vector into one that exists in 3D space
         ref_plane = Plane(self._plane.n, Point3D(0, 0, 0), self._plane.x)
@@ -999,8 +988,8 @@ class Face3D(Base2DIn3D):
             contours = [l_seg for l_seg in contours if l_seg.length >= tolerance]
         return contours
 
-    def countour_by_distance_between(self, distance, direction_vector=Vector2D(0, 1),
-                                     flip_side=False, tolerance=0):
+    def countour_by_distance_between(self, distance, direction_vector, flip_side,
+                                     tolerance):
         """Generate a list of LineSegment3D objects contouring the face.
 
         Args:
@@ -1009,13 +998,14 @@ class Face3D(Base2DIn3D):
                 are generated. This 2D vector will be interpreted into a 3D vector
                 within the plane of this Face. (0, 1) will usually generate
                 horizontal contours in 3D space, (1, 0) will generate vertical
-                contours, and (1, 1) will generate diagonal contours. Default: (0, 1).
+                contours, and (1, 1) will generate diagonal contours. Recommended
+                value is Vector2D(0, 1).
             flip_side: Boolean to note whether the side the contours start from
-                should be flipped. Default is False to have contours on top or right.
-                Setting to True will start contours on the bottom or left.
+                should be flipped. Recommended value is is False to have contours
+                start on top or right. Setting to True will start contours on
+                the bottom or left.
             tolerance: An optional value to remove any contours with a length less
-                than the tolerance. Default is 0, which will include all contours
-                no matter how small.
+                than the tolerance.
         """
         # interpret the 2D direction_vector into one that exists in 3D space
         ref_plane = Plane(self._plane.n, Point3D(0, 0, 0), self._plane.x)
@@ -1046,58 +1036,58 @@ class Face3D(Base2DIn3D):
             contours = [l_seg for l_seg in contours if l_seg.length >= tolerance]
         return contours
 
-    def countour_fins_by_number(self, fin_count, depth, offset=0, angle=0,
-                                contour_vector=Vector2D(0, 1), flip_side=False,
-                                tolerance=0):
+    def countour_fins_by_number(self, fin_count, depth, offset, angle,
+                                contour_vector, flip_side, tolerance):
         """Generate a list of Fac3D objects over this face (like louvers or fins).
 
         Args:
             fin_count: A positive integer for the number of fins to generate.
             depth: A number for the depth to extrude the fins.
             offset: A number for the distance to offset fins from this face.
-                Default is 0 for no offset.
+                Recommended value is 0 for no offset.
             angle: A number for the for an angle to rotate the fins in radians.
-                Default is 0 for no rotation.
+                Recommended value is 0 for no rotation.
             contour_vector: A Vector2D for the direction along which contours
                 are generated. This 2D vector will be interpreted into a 3D vector
                 within the plane of this Face. (0, 1) will usually generate
                 horizontal contours in 3D space, (1, 0) will generate vertical
-                contours, and (1, 1) will generate diagonal contours. Default: (0, 1).
+                contours, and (1, 1) will generate diagonal contours. Recommended
+                value is Vector2D(0, 1).
             flip_side: Boolean to note whether the side the fins start from
-                should be flipped. Default is False to have contours on top or right.
-                Setting to True will start contours on the bottom or left.
+                should be flipped. Recommended value is False to have contours
+                start on top or right. Setting to True will start contours on
+                the bottom or left.
             tolerance: An optional value to remove any contours with a length less
-                than the tolerance. Default is 0, which will include all contour fins
-                no matter how small.
+                than the tolerance.
         """
         extru_vec = self._get_fin_extrusion_vector(depth, angle, contour_vector)
         contours = self.countour_by_number(
             fin_count, contour_vector, flip_side, tolerance)
         return self._get_extrusion_fins(contours, extru_vec, offset)
 
-    def countour_fins_by_distance_between(self, distance, depth, offset=0, angle=0,
-                                          contour_vector=Vector2D(0, 1),
-                                          flip_side=False, tolerance=0):
+    def countour_fins_by_distance_between(self, distance, depth, offset, angle,
+                                          contour_vector, flip_side, tolerance):
         """Generate a list of Fac3D objects over this face (like louvers or fins).
 
         Args:
             distance: A number for the approximate distance between each contour.
             depth: A number for the depth to extrude the fins.
             offset: A number for the distance to offset fins from this face.
-                Default is 0 for no offset.
+                Recommended value is 0 for no offset.
             angle: A number for the for an angle to rotate the fins in radians.
-                Default is 0 for no rotation.
+                Recommended value is 0 for no rotation.
             contour_vector: A Vector2D for the direction along which contours
                 are generated. This 2D vector will be interpreted into a 3D vector
                 within the plane of this Face. (0, 1) will usually generate
                 horizontal contours in 3D space, (1, 0) will generate vertical
-                contours, and (1, 1) will generate diagonal contours. Default: (0, 1).
+                contours, and (1, 1) will generate diagonal contours. Recommended
+                value is Vector2D(0, 1).
             flip_side: Boolean to note whether the side the fins start from
-                should be flipped. Default is False to have contours on top or right.
-                Setting to True will start contours on the bottom or left.
+                should be flipped. Recommended value is False to have contours
+                start on top or right. Setting to True will start contours on
+                the bottom or left.
             tolerance: An optional value to remove any contours with a length less
-                than the tolerance. Default is 0, which will include all contour fins
-                no matter how small.
+                than the tolerance.
         """
         extru_vec = self._get_fin_extrusion_vector(depth, angle, contour_vector)
         contours = self.countour_by_distance_between(
@@ -1107,7 +1097,8 @@ class Face3D(Base2DIn3D):
     def sub_faces_by_ratio(self, ratio):
         """Get a list of faces with a combined area equal to the ratio times this face area.
 
-        All sub faces will lie inside the boundaries of this face.
+        All sub faces will lie inside the boundaries of this face and will have
+        the same normal as this face.
 
         Args:
             ratio: A number between 0 and 1 for the ratio between the area of
@@ -1128,6 +1119,54 @@ class Face3D(Base2DIn3D):
                     [pt.scale(scale_factor, _tri_mesh.face_centroids[i]) for pt in _tri])
             return [Face3D(_t, self.plane) for _t in _scaled_verts]
 
+    def sub_faces_by_ratio_gridded(self, ratio, x_dim, y_dim=None):
+        """Get a list of faces with a combined area equal to the ratio times this face area.
+
+        All sub faces will lie inside the boundaries of this face and have the same
+        normal as this face.
+
+        Sub faces will be arranged in a grid derived from this face's plane property.
+        Because the x_dim and y_dim refer to dimensions within the X and Y
+        coordinate system of this faces's plane, rotating this plane will
+        result in rotated grid cells.
+
+        If the x_dim and/or y_dim are too large for this face, this method will
+        return essentially the same result as the sub_faces_by_ratio method.
+
+        Args:
+            ratio: A number between 0 and 1 for the ratio between the area of
+                the sub faces and the area of this face.
+            x_dim: The x dimension of the grid cells as a number.
+            y_dim: The y dimension of the grid cells as a number. Default is None,
+                which will assume the same cell dimension for y as is set for x.
+
+        Returns:
+            A list of Face3D objects for sub faces.
+        """
+        try:  # get the gridded mesh derived from this face
+            grid_mesh = self.mesh_grid(x_dim, y_dim)
+        except AssertionError:  # there are no faces; just return sub_faces_by_ratio
+            return self.sub_faces_by_ratio(ratio)
+
+        # compute the area that each of the mesh faces need to be scaled to
+        _verts, _faces = grid_mesh.vertices, grid_mesh.faces
+        _x_dim = _verts[_faces[0][0]].distance_to_point(_verts[_faces[0][1]])
+        _y_dim = _verts[_faces[0][1]].distance_to_point(_verts[_faces[0][2]])
+        fac = (self.area * ratio) / (_x_dim * _y_dim * len(_faces))
+
+        # if the factor is greater than 1, sub-faces will be overlapping
+        if fac >= 1:
+            return self.sub_faces_by_ratio(ratio)
+        s_fac = fac ** 0.5
+
+        # generate the Face3D objects while scaling them to the correct size
+        sub_faces = []
+        for face, centr in zip(_faces, grid_mesh.face_centroids):
+            _f = Face3D(tuple(_verts[i].scale(s_fac, centr) for i in face), self.plane)
+            if self._is_sub_face(_f):  # catch edge cases
+                sub_faces.append(_f)
+        return sub_faces
+
     def sub_faces_by_ratio_rectangle(self, ratio, tolerance):
         """Get a list of faces with a combined area equal to the ratio times this face area.
 
@@ -1205,12 +1244,12 @@ class Face3D(Base2DIn3D):
         for face in other_faces:
             sub_faces.extend(face.sub_faces_by_ratio(ratio))
         return sub_faces
-    
+
     def sub_faces_by_dimension_rectangle(self, sub_rect_height, sub_rect_width,
                                          sill_height, horizontal_separation, tolerance):
-        """Get a list of rectangualr faces within this Face3D.
+        """Get a list of rectangular faces within this Face3D.
 
-        Note that this method will only yeild results if there is a rectangle to
+        Note that this method will only yield results if there is a rectangle to
         be extracted from this Face3D's geometry.
 
         Args:
@@ -1391,7 +1430,7 @@ class Face3D(Base2DIn3D):
                 The origin of this plane will be the lower left corner of the
                 rectangle and the X and Y axes will form the sides.
                 Default is the world XY plane.
-        
+
         Returns:
             A list of Face3D objects for sub faces.
         """
@@ -1451,7 +1490,7 @@ class Face3D(Base2DIn3D):
                     final_faces.append(Face3D(
                         (seg.p1, seg.p2, seg.p2 + h_vec, seg.p1 + h_vec), base_plane))
         else:
-            # make a single sub-rectangle at an apporporate sill height
+            # make a single sub-rectangle at an appropriate sill height
             max_sill_h = parent_height * 0.99 - (target_area / (parent_base * 0.98))
             sill_vec = base_plane.y * sill_height if sill_height < max_sill_h \
                 else base_plane.y * max_sill_h
@@ -1480,7 +1519,7 @@ class Face3D(Base2DIn3D):
                 final_faces = [Face3D((seg.p1, seg.p2, seg.p2 + h_vec, seg.p1 + h_vec),
                                       base_plane)]
         return final_faces
-    
+
     @staticmethod
     def sub_rects_from_rect_dimensions(
             base_plane, parent_base, parent_height, sub_rect_height, sub_rect_width,
@@ -1488,7 +1527,7 @@ class Face3D(Base2DIn3D):
         """Get a list of rectangular Face3D objects from dimensions and parameters.
 
         All of the resulting Face3D objects lie within a parent rectangle defined
-        by the parent_base, parent_height, and base_plane. 
+        by the parent_base, parent_height, and base_plane.
 
         Args:
             base_plane: A Plane object in which the rectangle exists.
@@ -1521,19 +1560,19 @@ class Face3D(Base2DIn3D):
         # adjust sill_hgt if sum of it and sub_rect_height > parent_height
         if sub_rect_height + sill_hgt >= parent_height:
             sill_hgt = parent_height - sub_rect_height - (parent_height * 0.01)
-        
+
         # ensure that the horizontal_separation is always greater than sub_rect_width
         if sub_rect_width >= horizontal_separation:
             horizontal_separation = sub_rect_width * 1.02
-        
-        # determine if the parameters should yeild multiple sub-windows or just one
+
+        # determine if the parameters should yield multiple sub-windows or just one
         max_width_break_up = parent_base / 2
         num_div = round(parent_base / horizontal_separation) if \
             parent_base > horizontal_separation / 2 else 1
         # properties used throughout the computation of sub-rectangles
         sill_vec = base_plane.y * sill_hgt
         bottom_seg = LineSegment3D.from_sdl(base_plane.o, base_plane.x, parent_base)
-        
+
         if sub_rect_width < max_width_break_up:
             # determine the number of times that the rectangle should be subdivided
             div_dist = parent_base / 2 if num_div == 1 else horizontal_separation
@@ -1551,16 +1590,16 @@ class Face3D(Base2DIn3D):
 
             # divide up the rectangle into points on the bottom
             btm_div_segs = tuple(LineSegment3D.from_end_points(pt, btm_div_pts[i + 1])
-                                for i, pt in enumerate(btm_div_pts[:-1]))
+                                 for i, pt in enumerate(btm_div_pts[:-1]))
             # scale the line segments along their center points
             line_cent_pt = tuple(line.point_at(0.5) for line in btm_div_segs)
             scale_factor = sub_rect_width / div_dist
             btm_div_segs = tuple(line.scale(scale_factor, mid_pt)
-                                for line, mid_pt in zip(btm_div_segs, line_cent_pt))
+                                 for line, mid_pt in zip(btm_div_segs, line_cent_pt))
             # generate the vertices by 'extruding' along window height vector
             h_vec = base_plane.y * sub_rect_height
             final_faces = [Face3D((line.p2, line.p1, line.p1 + h_vec, line.p2 + h_vec),
-                                base_plane) for line in btm_div_segs]
+                                  base_plane) for line in btm_div_segs]
         else:  # make a single sub-rectangle at an appropriate sill height
             if sub_rect_width >= parent_base:
                 sub_rect_width = parent_base * 0.98
@@ -1570,7 +1609,7 @@ class Face3D(Base2DIn3D):
             # generate the vertices by 'extruding' along window height vector
             h_vec = base_plane.y * sub_rect_height
             final_faces = [Face3D((seg.p2, seg.p1, seg.p1 + h_vec, seg.p2 + h_vec),
-                                base_plane)]
+                                  base_plane)]
         return final_faces
 
     def to_dict(self, include_plane=True, enforce_upper_left=False):
@@ -1701,7 +1740,7 @@ class Face3D(Base2DIn3D):
     def _remove_colinear(self, pts_3d, pts_2d, tolerance):
         """Remove colinear vertices from a list of Point2D.
 
-        This method determines colinearity by checking whether the area of the
+        This method determines co-linearity by checking whether the area of the
         triangle formed by 3 vertices is less than the tolerance.
         """
         new_vertices = []
@@ -1712,6 +1751,25 @@ class Face3D(Base2DIn3D):
                 new_vertices.append(pts_3d[i - 1])
         return new_vertices
 
+    def _is_sub_face(self, face):
+        """Check if a face is a sub-face of this face, bypassing coplanar check.
+
+        Args:
+            face: Another face for which sub-face equivalency will be tested.
+        """
+        verts2d = tuple(self.plane.xyz_to_xy(_v) for _v in face.vertices)
+        sub_poly = Polygon2D(verts2d)
+
+        if not self.has_holes:
+            return self.polygon2d.is_polygon_inside(sub_poly)
+        else:
+            if not self.boundary_polygon2d.is_polygon_inside(sub_poly):
+                return False
+            for hole_poly in self.hole_polygon2d:
+                if not hole_poly.is_polygon_outside(sub_poly):
+                    return False
+            return True
+
     def _vertices_between_points(self, start_pt, end_pt, tolerance):
         """Get the vertices between a start and end point.
 
@@ -1736,7 +1794,7 @@ class Face3D(Base2DIn3D):
             start = Point3D(diagonal.p1.x, diagonal.p2.y, diagonal.p1.z)
             end = Point3D(diagonal.p2.x, diagonal.p1.y, diagonal.p2.z)
             diagonal = LineSegment3D.from_end_points(start, end)
-        # flip if there's a horixzontal direction_vector to ensure always starts on left
+        # flip if there's a horizontal direction_vector to ensure always starts on left
         if direction_vector.x != 0 and self.normal.y > 0:
             diagonal = diagonal.flip()
         return diagonal
@@ -1823,35 +1881,47 @@ class Face3D(Base2DIn3D):
         return (close_pt_1, close_pt_2, close_pt_3, close_pt_4), other_faces
 
     @staticmethod
-    def _plane_from_vertices(vertices):
-        """Get a plane from a list of non-colinear vertices.
+    def _plane_from_vertices(verts):
+        """Get a plane from a list of vertices.
 
         Args:
-            vertices: The vertices to be used to extract the normal.
+            verts: The vertices to be used to extract the normal.
         """
         try:
-            normal = [0, 0, 0]
+            x1, y1, z1 = Face3D._normal_from_3pts(verts[-2], verts[-1], verts[0])
+            x2, y2, z2 = Face3D._normal_from_3pts(verts[-1], verts[0], verts[1])
+            normal = [x1 + x2, y1 + y2, z1 + z2]
             # walk around the whole shape to avoid colinear vertices
-            for i in range(len(vertices) - 2):
-                x, y, z = Face3D._normal_from_3pts(*vertices[i:i + 3])
+            for i in range(len(verts) - 2):
+                x, y, z = Face3D._normal_from_3pts(*verts[i:i + 3])
                 normal[0] += x
                 normal[1] += y
                 normal[2] += z
         except Exception as e:
             raise ValueError('Incorrect vertices input for Face3D:\n\t{}'.format(e))
-        return Plane(Vector3D(normal[0], normal[1], normal[2]), vertices[0])
+        return Plane(Vector3D(normal[0], normal[1], normal[2]), verts[0])
 
     @staticmethod
     def _normal_from_3pts(pt1, pt2, pt3):
         """Get a tuple representing a normal vector from 3 vertices.
 
         The vector will have a magnitude of 0 if vertices are colinear.
-        This method effectively performs the cross product of two vectors but
+        This method effectively performs the cross product of two unit vectors but
         the ladybug_geometry objects are not used in order to remove assertions
         and increase speed.
         """
+        # get two vectors for the two edges the 3 points form
         v1 = (pt2.x - pt1.x, pt2.y - pt1.y, pt2.z - pt1.z)
         v2 = (pt3.x - pt1.x, pt3.y - pt1.y, pt3.z - pt1.z)
+        # normalize the vectors to make them unit vectors
+        d1 = math.sqrt(v1[0] ** 2 + v1[1] ** 2 + v1[2] ** 2)
+        d2 = math.sqrt(v2[0] ** 2 + v2[1] ** 2 + v2[2] ** 2)
+        try:
+            v1 = (v1[0] / d1, v1[1] / d1, v1[2] / d1)
+            v2 = (v2[0] / d2, v2[1] / d2, v2[2] / d2)
+        except ZeroDivisionError:  # duplicate vertices in triplet
+            return (0, 0, 0)
+        # return the cross product
         return (v1[1] * v2[2] - v1[2] * v2[1],
                 -v1[0] * v2[2] + v1[2] * v2[0],
                 v1[0] * v2[1] - v1[1] * v2[0])
@@ -1878,7 +1948,7 @@ class Face3D(Base2DIn3D):
         _new_face._mesh2d = self._mesh2d
         _new_face._mesh3d = self._mesh3d
         return _new_face
-    
+
     def __key(self):
         """A tuple based on the object properties, useful for hashing."""
         return tuple(hash(pt) for pt in self._vertices) + (hash(self._plane),)
