@@ -84,7 +84,7 @@ class Vector3D(object):
         """Get the magnitude squared of the vector."""
         return self.x ** 2 + self.y ** 2 + self.z ** 2
 
-    def is_zero(self, tolerance=0):
+    def is_zero(self, tolerance):
         """Boolean to note whether the vector is within a given zero tolerance.
 
         Args:
@@ -208,7 +208,7 @@ class Vector3D(object):
 
     def __copy__(self):
         return self.__class__(self.x, self.y, self.z)
-    
+
     def __key(self):
         """A tuple based on the object properties, useful for hashing."""
         return (self.x, self.y, self.z)
@@ -413,7 +413,7 @@ class Point3D(Vector3D):
         """Get a point projected a point3d into a plane with a given normal and origin.
 
         Args:
-            normal: A Vector3D representing the normal vector of the plane into wich
+            normal: A Vector3D representing the normal vector of the plane into which
                 the plane will be projected. THIS VECTOR MUST BE NORMALIZED.
             origin: A Point3D representing the origin the plane into which the
                 point will be projected.
