@@ -9,10 +9,11 @@ from ladybug_geometry.geometry3d.ray import Ray3D
 from ladybug_geometry.geometry3d.face import Face3D
 
 import math
+import json
 
 
 def test_face3d_init():
-    """Test the initalization of Face3D objects and basic properties."""
+    """Test the initialization of Face3D objects and basic properties."""
     pts = (Point3D(0, 0, 2), Point3D(0, 2, 2), Point3D(2, 2, 2), Point3D(2, 0, 2))
     plane = Plane(Vector3D(0, 0, 1), Point3D(0, 0, 2))
     face = Face3D(pts, plane)
@@ -82,7 +83,7 @@ def test_face3d_to_from_dict():
 
 
 def test_face3d_init_from_vertices():
-    """Test the initalization of Face3D objects without a plane."""
+    """Test the initialization of Face3D objects without a plane."""
     pts = (Point3D(0, 0, 2), Point3D(0, 2, 2), Point3D(2, 2, 2), Point3D(2, 0, 2))
     face = Face3D(pts)
 
@@ -114,19 +115,19 @@ def test_face3d_init_from_vertices():
 
 
 def test_face3d_init_from_vertices_colinear():
-    """Test the initalization of Face3D objects with colinear vertices."""
+    """Test the initialization of Face3D objects with colinear vertices."""
     pts = (Point3D(0, 0, 2), Point3D(0, 1, 2), Point3D(0, 2, 2), Point3D(2, 2, 2),
            Point3D(2, 0, 2))
     face = Face3D(pts)
 
-    assert not face.normal.is_zero()
+    assert not face.normal.is_zero(0.000001)
     assert face.plane.n == Vector3D(0, 0, -1)
     assert face.plane.n == face.normal
     assert face.plane.o == Point3D(0, 0, 2)
 
 
 def test_face3d_init_from_extrusion():
-    """Test the initalization of Face3D from_extrusion."""
+    """Test the initialization of Face3D from_extrusion."""
     line_seg = LineSegment3D(Point3D(0, 0, 0), Vector3D(2, 0, 0))
     extru_vec = Vector3D(0, 0, 2)
     face = Face3D.from_extrusion(line_seg, extru_vec)
@@ -158,7 +159,7 @@ def test_face3d_init_from_extrusion():
 
 
 def test_face3d_init_from_rectangle():
-    """Test the initalization of Face3D from_rectangle."""
+    """Test the initialization of Face3D from_rectangle."""
     plane = Plane(Vector3D(0, 0, 1), Point3D(2, 2, 2))
     face = Face3D.from_rectangle(2, 2, plane)
 
@@ -188,7 +189,7 @@ def test_face3d_init_from_rectangle():
 
 
 def test_face3d_init_from_regular_polygon():
-    """Test the initalization of Face3D from_regular_polygon."""
+    """Test the initialization of Face3D from_regular_polygon."""
     plane = Plane(Vector3D(0, 0, 1), Point3D(2, 2, 2))
     face = Face3D.from_regular_polygon(8, 2, plane)
 
@@ -224,7 +225,7 @@ def test_face3d_init_from_regular_polygon():
 
 
 def test_face3d_init_from_shape_with_hole():
-    """Test the initalization of Face3D from_shape_with_holes with one hole."""
+    """Test the initialization of Face3D from_shape_with_holes with one hole."""
     bound_pts = [Point3D(0, 0), Point3D(4, 0), Point3D(4, 4), Point3D(0, 4)]
     hole_pts = [Point3D(1, 1), Point3D(3, 1), Point3D(3, 3), Point3D(1, 3)]
     face = Face3D(bound_pts, None, [hole_pts])
@@ -257,7 +258,7 @@ def test_face3d_init_from_shape_with_hole():
 
 
 def test_face3d_init_from_shape_with_holes():
-    """Test the initalization of Face3D from_shape_with_holes."""
+    """Test the initialization of Face3D from_shape_with_holes."""
     bound_pts = [Point3D(0, 0), Point3D(4, 0), Point3D(4, 4), Point3D(0, 4)]
     hole_pts_1 = [Point3D(1, 1), Point3D(1.5, 1), Point3D(1.5, 1.5), Point3D(1, 1.5)]
     hole_pts_2 = [Point3D(2, 2), Point3D(3, 2), Point3D(3, 3), Point3D(2, 3)]
@@ -288,7 +289,7 @@ def test_face3d_init_from_shape_with_holes():
 
 
 def test_face3d_init_from_punched_geometry():
-    """Test the initalization of Face3D from_shape_with_holes."""
+    """Test the initialization of Face3D from_shape_with_holes."""
     bound_pts = [Point3D(0, 0), Point3D(4, 0), Point3D(4, 4), Point3D(0, 4)]
     hole_pts_1 = [Point3D(1, 1), Point3D(1.5, 1), Point3D(1.5, 1.5), Point3D(1, 1.5)]
     hole_pts_2 = [Point3D(2, 2), Point3D(3, 2), Point3D(3, 3), Point3D(2, 3)]
@@ -321,6 +322,16 @@ def test_face3d_init_from_punched_geometry():
     assert face.is_self_intersecting is False
 
 
+def test_face3d_init_h_shape():
+    """Test the initialization of Face3D that is H-shaped."""
+    geo_file = './tests/json/h_shaped_face.json'
+    with open(geo_file, 'r') as fp:
+        geo_dict = json.load(fp)
+    floor_geo = Face3D.from_dict(geo_dict)
+
+    assert floor_geo.normal.z == pytest.approx(-1, rel=1e-3)
+
+
 def test_is_geometrically_equivalent():
     """Test the is_geometrically_equivalent method."""
     plane_1 = Plane(Vector3D(0, 0, 1))
@@ -759,7 +770,7 @@ def test_rotate():
     assert face.area == test_1.area
     assert len(face.vertices) == len(test_1.vertices)
 
-    test_2 = face.rotate(axis, math.pi/2, origin)
+    test_2 = face.rotate(axis, math.pi / 2, origin)
     assert test_2[0].x == pytest.approx(0, rel=1e-3)
     assert test_2[0].y == pytest.approx(-2, rel=1e-3)
     assert test_2[0].z == pytest.approx(0, rel=1e-3)
@@ -916,12 +927,12 @@ def test_project_point():
     assert face.project_point(pt_4) is None
 
 
-def test_get_mesh_grid():
-    """Test the Face3D get_mesh_grid method."""
+def test_mesh_grid():
+    """Test the Face3D mesh_grid method."""
     pts = (Point3D(0, 0), Point3D(0, 2), Point3D(2, 2), Point3D(2, 0))
     plane = Plane(Vector3D(0, 0, 1))
     face = Face3D(pts, plane)
-    mesh = face.get_mesh_grid(0.5)
+    mesh = face.mesh_grid(0.5)
 
     assert len(mesh.vertices) == 25
     assert len(mesh.faces) == 16
@@ -939,8 +950,8 @@ def test_get_mesh_grid():
     assert mesh.face_areas[0] == 0.25
     assert len(mesh.face_centroids) == 16
 
-    mesh_2 = face.get_mesh_grid(0.5, 0.5, 1, False)
-    mesh_3 = face.get_mesh_grid(0.5, 0.5, 1, True)
+    mesh_2 = face.mesh_grid(0.5, 0.5, 1, False)
+    mesh_3 = face.mesh_grid(0.5, 0.5, 1, True)
 
     assert mesh_2.min.z == pytest.approx(1, rel=1e-2)
     assert mesh_2.max.z == pytest.approx(1, rel=1e-2)
@@ -956,22 +967,22 @@ def test_countour_by_number():
     face_1 = Face3D(pts_1, plane)
     face_2 = Face3D(pts_2, plane)
 
-    contours = face_1.countour_by_number(4)
+    contours = face_1.countour_by_number(4, Vector2D(0, 1), False, 0.01)
     assert len(contours) == 4
     assert contours[0].p2.z == pytest.approx(2, rel=1e-3)
     assert contours[-1].p2.z == pytest.approx(0.5, rel=1e-3)
 
-    contours = face_1.countour_by_number(4, Vector2D(1))
+    contours = face_1.countour_by_number(4, Vector2D(1), False, 0.01)
     assert len(contours) == 4
     assert contours[-1].p2.x == pytest.approx(1.5, rel=1e-3)
 
-    contours = face_1.countour_by_number(4, Vector2D(1), True)
+    contours = face_1.countour_by_number(4, Vector2D(1), True, 0.01)
     assert len(contours) == 4
     assert contours[-1].p2.x == pytest.approx(0.5, rel=1e-3)
 
-    contours = face_2.countour_by_number(4)
+    contours = face_2.countour_by_number(4, Vector2D(0, 1), False, 0.01)
     assert len(contours) == 4
-    contours = face_2.countour_by_number(8, Vector2D(1))
+    contours = face_2.countour_by_number(8, Vector2D(1), False, 0.01)
     assert len(contours) == 8
 
 
@@ -983,22 +994,22 @@ def test_countour_by_distance_between():
     face_1 = Face3D(pts_1, plane)
     face_2 = Face3D(pts_2, plane)
 
-    contours = face_1.countour_by_distance_between(0.5)
+    contours = face_1.countour_by_distance_between(0.5, Vector2D(0, 1), False, 0.01)
     assert len(contours) == 4
     assert contours[0].p2.z == pytest.approx(2, rel=1e-3)
     assert contours[-1].p2.z == pytest.approx(0.5, rel=1e-3)
 
-    contours = face_1.countour_by_distance_between(0.5, Vector2D(1))
+    contours = face_1.countour_by_distance_between(0.5, Vector2D(1), False, 0.01)
     assert len(contours) == 4
     assert contours[-1].p2.x == pytest.approx(1.5, rel=1e-3)
 
-    contours = face_1.countour_by_distance_between(0.5, Vector2D(1), True)
+    contours = face_1.countour_by_distance_between(0.5, Vector2D(1), True, 0.01)
     assert len(contours) == 4
     assert contours[-1].p2.x == pytest.approx(0.5, rel=1e-3)
 
-    contours = face_2.countour_by_distance_between(0.5)
+    contours = face_2.countour_by_distance_between(0.5, Vector2D(0, 1), False, 0.01)
     assert len(contours) == 4
-    contours = face_2.countour_by_distance_between(0.5, Vector2D(1))
+    contours = face_2.countour_by_distance_between(0.5, Vector2D(1), False, 0.01)
     assert len(contours) == 8
 
 
@@ -1008,13 +1019,14 @@ def test_countour_fins_by_number():
     plane = Plane(Vector3D(0, 1, 0))
     face_1 = Face3D(pts_1, plane)
 
-    fins = face_1.countour_fins_by_number(4, 0.5, 0.5)
+    fins = face_1.countour_fins_by_number(4, 0.5, 0.5, 0, Vector2D(0, 1), False, 0.01)
     assert len(fins) == 4
 
-    fins = face_1.countour_fins_by_number(4, 0.5, 0.5, contour_vector=Vector2D(1))
+    fins = face_1.countour_fins_by_number(4, 0.5, 0.5, 0, Vector2D(1), False, 0.01)
     assert len(fins) == 4
 
-    fins = face_1.countour_fins_by_number(4, 0.5, 0.5, math.pi/4)
+    fins = face_1.countour_fins_by_number(
+        4, 0.5, 0.5, math.pi/4, Vector2D(0, 1), False, 0.01)
     assert len(fins) == 4
 
 
@@ -1026,15 +1038,16 @@ def test_countour_fins_by_distance_between():
     face_1 = Face3D(pts_1, plane)
     face_2 = Face3D(pts_2, plane)
 
-    fins = face_1.countour_fins_by_distance_between(0.5, 0.5, 0.5)
+    fins = face_1.countour_fins_by_distance_between(
+        0.5, 0.5, 0.5, 0, Vector2D(0, 1), False, 0.01)
     assert len(fins) == 4
 
-    fins = face_1.countour_fins_by_distance_between(0.25, 0.5, 0.5,
-                                                    contour_vector=Vector2D(1))
+    fins = face_1.countour_fins_by_distance_between(
+        0.25, 0.5, 0.5,0, Vector2D(1), False, 0.01)
     assert len(fins) == 8
 
-    fins = face_2.countour_fins_by_distance_between(0.5, 0.5, 0.5,
-                                                    contour_vector=Vector2D(1))
+    fins = face_2.countour_fins_by_distance_between(
+        0.5, 0.5, 0.5, 0, Vector2D(1), False, 0.01)
     assert len(fins) == 8
 
 
@@ -1100,6 +1113,20 @@ def test_extract_rectangle_complex():
     assert len(f1_result[2][0].vertices) == 4
 
 
+def test_sub_faces_by_ratio_gridded():
+    """Test the Face3D sub_faces_by_ratio_gridded method."""
+    pts_1 = (Point3D(0, 0, 0), Point3D(12, 0, 0), Point3D(12, 0, 12), Point3D(0, 0, 6))
+    face_1 = Face3D(pts_1)
+
+    sub_faces = face_1.sub_faces_by_ratio_gridded(0.4, 2, 2)
+    assert len(sub_faces) == 24
+    assert sum([face.area for face in sub_faces]) == pytest.approx(face_1.area * 0.4, rel=1e-3)
+
+    sub_faces = face_1.sub_faces_by_ratio_gridded(0.4, 12, 12)
+    assert len(sub_faces) == 1
+    assert sum([face.area for face in sub_faces]) == pytest.approx(face_1.area * 0.4, rel=1e-3)
+
+
 def test_sub_faces_by_ratio_rectangle():
     """Test the Face3D sub_faces_by_ratio_rectangle method."""
     pts_1 = (Point3D(0, 0, 0), Point3D(12, 0, 0), Point3D(12, 0, 2), Point3D(0, 0, 3))
